@@ -1,4 +1,3 @@
-
 #' LMTP Targeted Maximum Likelihood Estimator
 #'
 #' Targeted maximum likelihood estimator for the effects of traditional causal effects and
@@ -12,8 +11,8 @@
 #'  analysis, a vector containing the columns names of intermediate outcome variables and the final
 #'  outcome variable ordered by time. Only numeric values are allowed. If the outcome type
 #'  is binary, data should be coded as 0 and 1.
-#' @param baseline An optional vector of columns names of baseline covariates to be
-#'  included for adjustment at every timepoint.
+#' @param baseline An optional vector containing the column names of baseline covariates to be
+#'  included for adjustment at every time point.
 #' @param time_vary A list the same length as the number of time points of observation with
 #'  the column names for new time-varying covariates introduced at each time point. The list
 #'  should be ordered following the time ordering of the model.
@@ -25,19 +24,28 @@
 #' @param k An integer specifying how previous time points should be
 #'  used for estimation at the given time point. Default is \code{Inf},
 #'  all time points.
-#' @param outcome_type Outcome variable type (i.e., continuous, binomial).
+#' @param outcome_type Outcome variable type (i.e., continuous, binomial, survival).
 #' @param id An optional column name containing cluster level identifiers.
 #' @param bounds An optional vector of the bounds for continuous outcomes. If \code{NULL},
 #'  the bounds will be taken as the minimum and maximum of the observed data.
 #'  Should be left as \code{NULL} if the outcome type is binary.
-#' @param learners_outcome An optional \code{sl3} learner stack for estimation of the outcome
-#'  regression. If not specified, will default to using a generalized linear model.
-#' @param learners_trt An optional \code{sl3} learner stack for estimation of the exposure
-#'  mechanism. If not specified, will default to using a generalized linear model.
+#' @param learners_outcome A vector of \code{SuperLearner} algorithms for estimation
+#'  of the outcome regression. Default is \code{"SL.glm"}, a main effects GLM.
+#' @param learners_trt A vector of \code{SuperLearner} algorithms for estimation
+#'  of the exposure mechanism. Default is \code{"SL.glm"}, a main effects GLM.
 #' @param folds The number of folds to be used for cross-fitting. Minimum allowable number
 #' is two folds.
-#' @param bound Determines that maximum and minimum values (scaled) predictions
+#' @param weights An optional vector of length n containing sampling weights.
+#' @param return_all_ratios Logical. If \code{TRUE}, the non-cumulative product density
+#'  ratios will be returned. The default is \code{FALSE}.
+#' @param .bound Determines that maximum and minimum values (scaled) predictions
 #'  will be bounded by. The default is 1e-5, bounding predictions by 1e-5 and 0.9999.
+#' @param .trim Determines the amount the density ratios should be trimmed.
+#'  The default is 0.999, trimming the density ratios greater than the 0.999 percentile
+#'  to the 0.999 percentile. A value of 1 indicates no trimming.
+#' @param .SL_folds Integer. Controls the number of splits to be used for fitting
+#'  the Super Learner. The default is 10.
+
 #' @return A list of class \code{lmtp} containing the following components:
 #'
 #' \item{estimator}{The estimator used, in this case "TMLE".}
@@ -50,6 +58,8 @@
 #' \item{outcome_reg}{An n x Tau + 1 matrix of outcome regression predictions.
 #'  The mean of the first column is used for calculating theta.}
 #' \item{density_ratios}{An n x Tau matrix of the estimated density ratios.}
+#' \item{raw_ratios}{An n x Tau matrix of the estimated non-cumulative product density ratios.
+#'  \code{NULL} if \code{return_all_ratios = FALSE}.}
 #' \item{weights_m}{A list the same length as \code{folds}, containing the Super Learner
 #'  ensemble weights at each time-point for each fold for the outcome regression.}
 #' \item{weights_r}{A list the same length as \code{folds}, containing the Super Learner
@@ -60,12 +70,11 @@
 #' @export
 lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
                       time_vary = NULL, cens = NULL, shift, k = Inf,
-                      outcome_type = c("binomial", "continuous"), id = NULL,
-                      bounds = NULL, learners_outcome = NULL,
-                      learners_trt = NULL, folds = 10, bound = 1e-5) {
-
-  # setup -------------------------------------------------------------------
-
+                      outcome_type = c("binomial", "continuous", "survival"),
+                      id = NULL, bounds = NULL, learners_outcome = "SL.glm",
+                      learners_trt = "SL.glm", folds = 10, weights = NULL,
+                      return_all_ratios = FALSE,
+                      .bound = 1e-5, .trim = 0.999, .SL_folds = 10) {
   meta <- Meta$new(
     data = data,
     trt = trt,
@@ -80,43 +89,38 @@ lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
     id = id,
     outcome_type = match.arg(outcome_type),
     V = folds,
+    weights = weights,
     bounds = bounds,
-    bound = bound
+    bound = .bound
   )
 
   pb <- progressr::progressor(meta$tau*folds*2)
 
-  # propensity --------------------------------------------------------------
-
-  dens_ratio <- ratio_dr(
-    cf_r(meta$data, shift, folds, meta$trt, cens, meta$determ, meta$tau,
-         meta$node_list$trt, learners_trt, pb, meta$weights_r),
-    folds
-  )
-
-  # tmle --------------------------------------------------------------------
+  ratios <- cf_r(meta$data, shift, folds, meta$trt, cens, meta$risk, meta$tau,
+                 meta$node_list$trt, learners_trt, pb, meta$weights_r, .SL_folds)
+  cumprod_ratios <- ratio_dr(ratios, folds, .trim)
 
   estims <-
     cf_tmle(meta$data, meta$shifted_data, folds, "xyz", meta$node_list$outcome,
-            cens, meta$determ, meta$tau, meta$outcome_type, meta$m, meta$m,
-            dens_ratio, learners_outcome, pb, meta$weights_m)
-
-  # return estimates --------------------------------------------------------
+            cens, meta$risk, meta$tau, meta$outcome_type, meta$m, meta$m,
+            cumprod_ratios, learners_outcome, pb, meta$weights, meta$weights_m, .SL_folds)
 
   out <- compute_theta(
-    estimator = "dr",
+    estimator = "tml",
     eta = list(
       estimator = "TMLE",
       m = estims,
-      r = recombine_dens_ratio(dens_ratio),
+      r = recombine_dens_ratio(cumprod_ratios),
+      raw_ratios = if (return_all_ratios) recombine_raw_ratio(ratios),
       tau = meta$tau,
       folds = meta$folds,
       id = meta$id,
       outcome_type = meta$outcome_type,
       bounds = meta$bounds,
+      weights = weights,
       shift = deparse(substitute((shift))),
       weights_m = pluck_weights("m", estims),
-      weights_r = pluck_weights("r", dens_ratio),
+      weights_r = pluck_weights("r", cumprod_ratios),
       outcome_type = meta$outcome_type
     ))
 
@@ -136,8 +140,8 @@ lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
 #'  analysis, a vector containing the columns names of intermediate outcome variables and the final
 #'  outcome variable ordered by time. Only numeric values are allowed. If the outcome type
 #'  is binary, data should be coded as 0 and 1.
-#' @param baseline An optional vector of columns names of baseline covariates to be
-#'  included for adjustment at every timepoint.
+#' @param baseline An optional vector containing the column names of baseline covariates to be
+#'  included for adjustment at every time point.
 #' @param time_vary A list the same length as the number of time points of observation with
 #'  the column names for new time-varying covariates introduced at each time point. The list
 #'  should be ordered following the time ordering of the model.
@@ -149,19 +153,28 @@ lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
 #' @param k An integer specifying how previous time points should be
 #'  used for estimation at the given time point. Default is \code{Inf},
 #'  all time points.
-#' @param outcome_type Outcome variable type (i.e., continuous, binomial).
+#' @param outcome_type Outcome variable type (i.e., continuous, binomial, survival).
 #' @param id An optional column name containing cluster level identifiers.
 #' @param bounds An optional vector of the bounds for continuous outcomes. If \code{NULL},
 #'  the bounds will be taken as the minimum and maximum of the observed data.
 #'  Should be left as \code{NULL} if the outcome type is binary.
-#' @param learners_outcome An optional \code{sl3} learner stack for estimation of the outcome
-#'  regression. If not specified, will default to using a generalized linear model.
-#' @param learners_trt An optional \code{sl3} learner stack for estimation of the exposure
-#'  mechanism. If not specified, will default to using a generalized linear model.
+#' @param learners_outcome A vector of \code{SuperLearner} algorithms for estimation
+#'  of the outcome regression. Default is \code{"SL.glm"}, a main effects GLM.
+#' @param learners_trt A vector of \code{SuperLearner} algorithms for estimation
+#'  of the exposure mechanism. Default is \code{"SL.glm"}, a main effects GLM.
 #' @param folds The number of folds to be used for cross-fitting. Minimum allowable number
 #' is two folds.
-#' @param bound Determines that maximum and minimum values (scaled) predictions
+#' @param weights An optional vector of length n containing sampling weights.
+#' @param return_all_ratios Logical. If \code{TRUE}, the non-cumulative product density
+#'  ratios will be returned. The default is \code{FALSE}.
+#' @param .bound Determines that maximum and minimum values (scaled) predictions
 #'  will be bounded by. The default is 1e-5, bounding predictions by 1e-5 and 0.9999.
+#' @param .trim Determines the amount the density ratios should be trimmed.
+#'  The default is 0.999, trimming the density ratios greater than the 0.999 percentile
+#'  to the 0.999 percentile. A value of 1 indicates no trimming.
+#' @param .SL_folds Integer. Controls the number of splits to be used for fitting
+#'  the Super Learner. The default is 10.
+#'
 #' @return A list of class \code{lmtp} containing the following components:
 #'
 #' \item{estimator}{The estimator used, in this case "SDR".}
@@ -174,6 +187,8 @@ lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
 #' \item{outcome_reg}{An n x Tau + 1 matrix of outcome regression predictions.
 #'  The mean of the first column is used for calculating theta.}
 #' \item{density_ratios}{An n x Tau matrix of the estimated density ratios.}
+#' \item{raw_ratios}{An n x Tau matrix of the estimated non-cumulative product density ratios.
+#'  \code{NULL} if \code{return_all_ratios = FALSE}.}
 #' \item{weights_m}{A list the same length as \code{folds}, containing the Super Learner
 #'  ensemble weights at each time-point for each fold for the outcome regression.}
 #' \item{weights_r}{A list the same length as \code{folds}, containing the Super Learner
@@ -184,12 +199,11 @@ lmtp_tmle <- function(data, trt, outcome, baseline = NULL,
 #' @example inst/examples/sdr-ex.R
 lmtp_sdr <- function(data, trt, outcome, baseline = NULL,
                      time_vary = NULL, cens = NULL, shift, k = Inf,
-                     outcome_type = c("binomial", "continuous"), id = NULL,
-                     bounds = NULL, learners_outcome = NULL,
-                     learners_trt = NULL, folds = 10, bound = 1e-5) {
-
-  # setup -------------------------------------------------------------------
-
+                     outcome_type = c("binomial", "continuous", "survival"),
+                     id = NULL, bounds = NULL, learners_outcome = "SL.glm",
+                     learners_trt = "SL.glm", folds = 10, weights = NULL,
+                     return_all_ratios = FALSE,
+                     .bound = 1e-5, .trim = 0.999, .SL_folds = 10) {
   meta <- Meta$new(
     data = data,
     trt = trt,
@@ -204,40 +218,37 @@ lmtp_sdr <- function(data, trt, outcome, baseline = NULL,
     id = id,
     outcome_type = match.arg(outcome_type),
     V = folds,
+    weights = NULL,
     bounds = bounds,
-    bound = bound
+    bound = .bound
   )
 
   pb <- progressr::progressor(meta$tau*folds*2)
 
-  # propensity --------------------------------------------------------------
-
-  raw_ratio <- cf_r(meta$data, shift, folds, meta$trt, cens, meta$determ, meta$tau,
-                    meta$node_list$trt, learners_trt, pb, meta$weights_r)
-
-  # sdr ---------------------------------------------------------------------
+  ratios <- cf_r(meta$data, shift, folds, meta$trt, cens, meta$risk, meta$tau,
+                 meta$node_list$trt, learners_trt, pb, meta$weights_r, .SL_folds)
 
   estims <-
     cf_sdr(meta$data, meta$shifted_data, folds, "xyz", meta$node_list$outcome,
-           cens, meta$determ, meta$tau, meta$outcome_type, meta$m, meta$m,
-           raw_ratio, learners_outcome, pb, meta$weights_m)
-
-  # return estimates --------------------------------------------------------
+           cens, meta$risk, meta$tau, meta$outcome_type, meta$m, meta$m,
+           ratios, learners_outcome, pb, meta$weights_m, .trim, .SL_folds)
 
   out <- compute_theta(
-    estimator = "dr",
+    estimator = "sdr",
     eta = list(
       estimator = "SDR",
       m = estims,
-      r = recombine_dens_ratio(ratio_dr(raw_ratio, folds)),
+      r = recombine_dens_ratio(ratio_dr(ratios, folds, .trim)),
+      raw_ratios = if (return_all_ratios) recombine_raw_ratio(ratios),
       tau = meta$tau,
       folds = meta$folds,
+      weights = weights,
       id = meta$id,
       outcome_type = meta$outcome_type,
       bounds = meta$bounds,
       shift = deparse(substitute((shift))),
       weights_m = pluck_weights("m", estims),
-      weights_r = pluck_weights("r", raw_ratio),
+      weights_r = pluck_weights("r", ratios),
       outcome_type = meta$outcome_type
     ))
 
@@ -257,8 +268,8 @@ lmtp_sdr <- function(data, trt, outcome, baseline = NULL,
 #'  analysis, a vector containing the columns names of intermediate outcome variables and the final
 #'  outcome variable ordered by time. Only numeric values are allowed. If the outcome type
 #'  is binary, data should be coded as 0 and 1.
-#' @param baseline An optional vector of columns names of baseline covariates to be
-#'  included for adjustment at every timepoint.
+#' @param baseline An optional vector containing the column names of baseline covariates to be
+#'  included for adjustment at every time point.
 #' @param time_vary A list the same length as the number of time points of observation with
 #'  the column names for new time-varying covariates introduced at each time point. The list
 #'  should be ordered following the time ordering of the model.
@@ -270,17 +281,20 @@ lmtp_sdr <- function(data, trt, outcome, baseline = NULL,
 #' @param k An integer specifying how previous time points should be
 #'  used for estimation at the given time point. Default is \code{Inf},
 #'  all time points.
-#' @param outcome_type Outcome variable type (i.e., continuous, binomial).
+#' @param outcome_type Outcome variable type (i.e., continuous, binomial, survival).
 #' @param id An optional column name containing cluster level identifiers.
 #' @param bounds An optional vector of the bounds for continuous outcomes. If \code{NULL},
 #'  the bounds will be taken as the minimum and maximum of the observed data.
 #'  Should be left as \code{NULL} if the outcome type is binary.
-#' @param learners An optional \code{sl3} learner stack for estimation of the outcome
-#'  regression. If not specified, will default to using a generalized linear model.
+#' @param learners A vector of \code{SuperLearner} algorithms for estimation
+#'  of the outcome regression. Default is \code{"SL.glm"}, a main effects GLM.
 #' @param folds The number of folds to be used for cross-fitting. Minimum allowable number
 #'  is two folds.
-#' @param bound Determines that maximum and minimum values (scaled) predictions
+#' @param weights An optional vector of length n containing sampling weights.
+#' @param .bound Determines that maximum and minimum values (scaled) predictions
 #'  will be bounded by. The default is 1e-5, bounding predictions by 1e-5 and 0.9999.
+#' @param .SL_folds Integer. Controls the number of splits to be used for fitting
+#'  the Super Learner. The default is 10.
 #'
 #' @return A list of class \code{lmtp} containing the following components:
 #'
@@ -300,11 +314,9 @@ lmtp_sdr <- function(data, trt, outcome, baseline = NULL,
 #' @example inst/examples/sub-ex.R
 lmtp_sub <- function(data, trt, outcome, baseline = NULL,
                      time_vary = NULL, cens = NULL, shift, k = Inf,
-                     outcome_type = c("binomial", "continuous"), id = NULL,
-                     bounds = NULL, learners = NULL, folds = 10, bound = 1e-5) {
-
-  # setup -------------------------------------------------------------------
-
+                     outcome_type = c("binomial", "continuous", "survival"),
+                     id = NULL, bounds = NULL, learners = "SL.glm", folds = 10,
+                     weights = NULL, .bound = 1e-5, .SL_folds = 10) {
   meta <- Meta$new(
     data = data,
     trt = trt,
@@ -319,19 +331,17 @@ lmtp_sub <- function(data, trt, outcome, baseline = NULL,
     id = id,
     outcome_type = match.arg(outcome_type),
     V = folds,
+    weights = NULL,
     bounds = bounds,
-    bound = bound
+    bound = .bound
   )
 
   pb <- progressr::progressor(meta$tau*folds)
 
-  # substitution ------------------------------------------------------------
-
-  estims <- cf_sub(meta$data, meta$shifted_data, folds, "xyz", meta$node_list$outcome,
-                   cens, meta$determ, meta$tau, meta$outcome_type,
-                   learners, meta$m, pb, meta$weights_m)
-
-  # return estimates --------------------------------------------------------
+  estims <- cf_sub(meta$data, meta$shifted_data, folds, "xyz",
+                   meta$node_list$outcome,
+                   cens, meta$risk, meta$tau, meta$outcome_type,
+                   learners, meta$m, pb, meta$weights_m, .SL_folds)
 
   out <- compute_theta(
     estimator = "sub",
@@ -339,13 +349,14 @@ lmtp_sub <- function(data, trt, outcome, baseline = NULL,
       m = estims$m,
       outcome_type = meta$outcome_type,
       bounds = meta$bounds,
+      folds = meta$folds,
+      weights = weights,
       shift = deparse(substitute((shift))),
       weights_m = pluck_weights("m", estims),
       outcome_type = meta$outcome_type
     ))
 
   return(out)
-
 }
 
 #' LMTP IPW Estimator
@@ -361,8 +372,8 @@ lmtp_sub <- function(data, trt, outcome, baseline = NULL,
 #'  analysis, a vector containing the columns names of intermediate outcome variables and the final
 #'  outcome variable ordered by time. Only numeric values are allowed. If the outcome type
 #'  is binary, data should be coded as 0 and 1.
-#' @param baseline An optional vector of columns names of baseline covariates to be
-#'  included for adjustment at every timepoint.
+#' @param baseline An optional vector containing the column names of baseline covariates to be
+#'  included for adjustment at every time point.
 #' @param time_vary A list the same length as the number of time points of observation with
 #'  the column names for new time-varying covariates introduced at each time point. The list
 #'  should be ordered following the time ordering of the model.
@@ -371,16 +382,25 @@ lmtp_sub <- function(data, trt, outcome, baseline = NULL,
 #'  present or if time-to-event outcome, must be provided.
 #' @param shift A two argument function that specifies how treatment variables should be shifted.
 #'  See examples for how to specify shift functions for continuous, binary, and categorical exposures.
+#' @param outcome_type Outcome variable type (i.e., continuous, binomial, survival).
 #' @param k An integer specifying how previous time points should be
 #'  used for estimation at the given time point. Default is \code{Inf},
 #'  all time points.
 #' @param id An optional column name containing cluster level identifiers.
-#' @param learners An optional \code{sl3} learner stack for estimation of the exposure
-#'  mechanism. If not specified, will default to using a generalized linear model.
+#' @param learners A vector of \code{SuperLearner} algorithms for estimation
+#'  of the exposure mechanism. Default is \code{"SL.glm"}, a main effects GLM.
 #' @param folds The number of folds to be used for cross-fitting. Minimum allowable number
 #'  is two folds.
-#' @param bound Determines that maximum and minimum values (scaled) predictions
+#' @param weights An optional vector of length n containing sampling weights.
+#' @param return_all_ratios Logical. If \code{TRUE}, the non-cumulative product density
+#'  ratios will be returned. The default is \code{FALSE}.
+#' @param .bound Determines that maximum and minimum values (scaled) predictions
 #'  will be bounded by. The default is 1e-5, bounding predictions by 1e-5 and 0.9999.
+#' @param .trim Determines the amount the density ratios should be trimmed.
+#'  The default is 0.999, trimming the density ratios greater than the 0.999 percentile
+#'  to the 0.999 percentile. A value of 1 indicates no trimming.
+#' @param .SL_folds Integer. Controls the number of splits to be used for fitting
+#'  the Super Learner. The default is 10.
 #'
 #' @return A list of class \code{lmtp} containing the following components:
 #'
@@ -391,17 +411,19 @@ lmtp_sub <- function(data, trt, outcome, baseline = NULL,
 #' \item{high}{NA}
 #' \item{shift}{The shift function specifying the treatment policy of interest.}
 #' \item{density_ratios}{An n x Tau matrix of the estimated density ratios.}
+#' \item{raw_ratios}{An n x Tau matrix of the estimated non-cumulative product density ratios.
+#'  \code{NULL} if \code{return_all_ratios = FALSE}.}
 #' \item{weights_r}{A list the same length as \code{folds}, containing the Super Learner
 #'  ensemble weights at each time-point for each fold for the propensity.}
 #' @export
 #'
 #' @example inst/examples/ipw-ex.R
 lmtp_ipw <- function(data, trt, outcome, baseline = NULL,
-                     time_vary = NULL, cens = NULL, k = Inf, id = NULL, shift,
-                     learners = NULL, folds = 10, bound = 1e-5) {
-
-  # setup -------------------------------------------------------------------
-
+                     time_vary = NULL, cens = NULL, k = Inf,
+                     id = NULL, shift, outcome_type = c("binomial", "continuous", "survival"),
+                     learners = "SL.glm", folds = 10, weights = NULL,
+                     return_all_ratios = FALSE,
+                     .bound = 1e-5, .trim = 0.999, .SL_folds = 10) {
   meta <- Meta$new(
     data = data,
     trt = trt,
@@ -414,38 +436,36 @@ lmtp_ipw <- function(data, trt, outcome, baseline = NULL,
     learners_trt = learners,
     learners_outcome = NULL,
     id = id,
-    outcome_type = NULL,
+    outcome_type = match.arg(outcome_type),
     V = folds,
+    weights = NULL,
     bounds = NULL,
-    bound = bound
+    bound = .bound
   )
 
   pb <- progressr::progressor(meta$tau*folds)
 
-  # propensity --------------------------------------------------------------
-
-  dens_ratio <-
-    ratio_ipw(
-      recombine_ipw(
-        cf_r(meta$data, shift, folds, meta$trt, cens, meta$deterministic,
-             meta$tau, meta$node_list$trt, learners, pb, meta$weights_r
-        )
-      )
-    )
-
-  # return estimates --------------------------------------------------------
+  ratios <- cf_r(meta$data, shift, folds, meta$trt, cens, meta$risk,
+                 meta$tau, meta$node_list$trt, learners, pb, meta$weights_r,
+                 .SL_folds)
+  cumprod_ratios <- ratio_ipw(recombine_ipw(ratios), .trim)
 
   out <- compute_theta(
     estimator = "ipw",
     eta = list(
-      r = dens_ratio$r,
-      y = data[[final_outcome(outcome)]],
+      r = cumprod_ratios$r,
+      raw_ratios = if (return_all_ratios) recombine_raw_ratio(ratios),
+      y = if (meta$survival) {
+        convert_to_surv(data[[final_outcome(outcome)]])
+      } else {
+        data[[final_outcome(outcome)]]
+      },
       folds = meta$folds,
+      weights = weights,
       tau = meta$tau,
       shift = deparse(substitute((shift))),
-      weights_r = dens_ratio$sl_weights
+      weights_r = cumprod_ratios$sl_weights
     ))
 
   return(out)
-
 }
